@@ -129,6 +129,19 @@ static const cairo_font_face_backend_t _cairo_atsui_font_face_backend = {
     _cairo_atsui_font_face_scaled_font_create
 };
 
+/**
+ * cairo_atsui_font_face_create_for_atsu_font_id
+ * @font_id: an ATSUFontID for the font.
+ *
+ * Creates a new font for the ATSUI font backend based on an
+ * #ATSUFontID. This font can then be used with
+ * cairo_set_font_face() or cairo_scaled_font_create().
+ *
+ * Return value: a newly created #cairo_font_face_t. Free with
+ *  cairo_font_face_destroy() when you are done using it.
+ *
+ * Since: 1.4
+ **/
 cairo_font_face_t *
 cairo_atsui_font_face_create_for_atsu_font_id (ATSUFontID font_id)
 {
@@ -147,14 +160,6 @@ cairo_atsui_font_face_create_for_atsu_font_id (ATSUFontID font_id)
     return &font_face->base;
 }
 
-static CGAffineTransform
-CGAffineTransformMakeWithCairoFontScale(const cairo_matrix_t *scale)
-{
-    return CGAffineTransformMake(scale->xx, scale->yx,
-                                 scale->xy, scale->yy,
-                                 0, 0);
-}
-
 static ATSUStyle
 CreateSizedCopyOfStyle(ATSUStyle inStyle, 
 		       const Fixed *theSize, 
@@ -399,50 +404,6 @@ _cairo_atsui_font_fini(void *abstract_font)
         ATSUDisposeStyle(font->unscaled_style);
 }
 
-static
-OSStatus _move_to_for_metrics (const Float32Point *point, void *callback_data)
-{
-    CGMutablePathRef path = callback_data;
-
-    CGPathMoveToPoint (path, &CGAffineTransformIdentity,
-			   point->x, point->y);
-    return noErr;
-}
-
-static
-OSStatus _line_to_for_metrics(const Float32Point *point, void *callback_data)
-{
-    CGMutablePathRef path = callback_data;
-
-    CGPathAddLineToPoint (path, &CGAffineTransformIdentity,
-			   point->x, point->y);
-    return noErr;
-}
-
-static
-OSStatus _curve_to_for_metrics (const Float32Point *point1,
-				const Float32Point *point2,
-				const Float32Point *point3,
-				void *callback_data)
-{
-    CGMutablePathRef path = callback_data;
-
-    CGPathAddCurveToPoint (path, &CGAffineTransformIdentity,
-			   point1->x, point1->y,
-			   point2->x, point2->y,
-			   point3->x, point3->y);
-    return noErr;
-}
-
-static
-OSStatus _close_path_for_metrics(void *callback_data)
-{
-    CGMutablePathRef path = callback_data;
-
-    CGPathCloseSubpath (path);
-    return noErr;
-}
-
 static GlyphID 
 _cairo_atsui_scaled_glyph_index (cairo_scaled_glyph_t *scaled_glyph) {
     unsigned long index = _cairo_scaled_glyph_index (scaled_glyph);
@@ -456,16 +417,10 @@ _cairo_atsui_font_init_glyph_metrics (cairo_atsui_font_t *scaled_font,
 				      cairo_scaled_glyph_t *scaled_glyph)
 {
     cairo_text_extents_t extents = {0, 0, 0, 0, 0, 0};
-    OSStatus err, callback_err;
+    OSStatus err;
     ATSGlyphScreenMetrics metricsH;
-    static ATSCubicMoveToUPP moveProc = NULL;
-    static ATSCubicLineToUPP lineProc = NULL;
-    static ATSCubicCurveToUPP curveProc = NULL;
-    static ATSCubicClosePathUPP closePathProc = NULL;
-    CGMutablePathRef path;
     GlyphID theGlyph = _cairo_atsui_scaled_glyph_index (scaled_glyph);
     double xscale, yscale;
-    CGRect rect;
 
     if (theGlyph == kATSDeletedGlyphcode) {
 	_cairo_scaled_glyph_set_metrics (scaled_glyph,
@@ -491,41 +446,15 @@ _cairo_atsui_font_init_glyph_metrics (cairo_atsui_font_t *scaled_font,
 
     extents.x_advance = metricsH.deviceAdvance.x * xscale;
     extents.y_advance = 0;
-
-    if (moveProc == NULL) {
-        moveProc = NewATSCubicMoveToUPP (_move_to_for_metrics);
-        lineProc = NewATSCubicLineToUPP (_line_to_for_metrics);
-        curveProc = NewATSCubicCurveToUPP (_curve_to_for_metrics);
-        closePathProc = NewATSCubicClosePathUPP (_close_path_for_metrics);
-    }
-
-    path = CGPathCreateMutable ();
-
-    /* The callback error contains any error our functions returned.
-     * Its only meaningful if err != noErr, and we don't currently
-     * use it for anything.
-     */
-    err = ATSUGlyphGetCubicPaths (scaled_font->style, theGlyph,
-				  moveProc, lineProc, curveProc, closePathProc,
-				  (void *)path, &callback_err);
-
-    if (err != noErr) {
-	CGPathRelease (path);
-	return CAIRO_STATUS_NO_MEMORY;
-    }
-
-    rect = CGPathGetBoundingBox (path);
-
-    extents.x_bearing = rect.origin.x * xscale;
-    extents.y_bearing = rect.origin.y * yscale;
-    extents.width = rect.size.width * xscale;
-    extents.height = rect.size.height * yscale;
+    
+    extents.x_bearing = metricsH.topLeft.x * xscale;
+    extents.y_bearing = -metricsH.topLeft.y * yscale;
+    extents.width = metricsH.width * xscale;
+    extents.height = metricsH.height * yscale;
 
     _cairo_scaled_glyph_set_metrics (scaled_glyph,
 				     &scaled_font->base,
 				     &extents);
-    CGPathRelease (path);
-
     return CAIRO_STATUS_SUCCESS;
 }
 
@@ -669,6 +598,7 @@ _cairo_atsui_scaled_font_init_glyph_surface (cairo_atsui_font_t *scaled_font,
     CGContextRef drawingContext;
     cairo_image_surface_t *surface;
     cairo_format_t format;
+    cairo_status_t status;
 
     ATSFontRef atsFont;
     CGFontRef cgFont;
@@ -685,8 +615,9 @@ _cairo_atsui_scaled_font_init_glyph_surface (cairo_atsui_font_t *scaled_font,
 
     if (theGlyph == kATSDeletedGlyphcode) {
 	surface = (cairo_image_surface_t *)cairo_image_surface_create (CAIRO_FORMAT_A8, 2, 2);
-	if (cairo_surface_status (surface))
-	    return cairo_surface_status (surface);
+	status = cairo_surface_status ((cairo_surface_t *)surface);
+	if (status)
+	    return status;
 
 	_cairo_scaled_glyph_set_surface (scaled_glyph,
 					 &base,
@@ -752,8 +683,9 @@ _cairo_atsui_scaled_font_init_glyph_surface (cairo_atsui_font_t *scaled_font,
 
     /* create the glyph mask surface */
     surface = (cairo_image_surface_t *)cairo_image_surface_create (format, bbox.size.width, bbox.size.height);
-    if (cairo_surface_status (surface))
-	return cairo_surface_status (surface);
+    status = cairo_surface_status ((cairo_surface_t *)surface);
+    if (status)
+	return status;
 
     /* Create a CGBitmapContext for the dest surface for drawing into */
     {
