@@ -127,6 +127,11 @@ gdip_get_display_dpi ()
 	if (dpis == 0) {
 		char *val;
 
+		if (getenv ("MONO_MWF_USE_QUARTZ_BACKEND") != NULL) {
+			dpis = 96.0f;
+			return dpis;
+		}
+
 		display = XOpenDisplay (0);
 		/* If the display is openable lets try to read dpi from it; otherwise use a default of 96.0f */
 		if (display) {
