@@ -254,22 +254,16 @@ GdipCreateFromHWND (void *hwnd, GpGraphics **graphics)
 #endif
 
 #ifdef CAIRO_HAS_QUARTZ_SURFACE
-
-/* This is a dirty hack; but it gets us around header include issues for now
- FIXME: We need to split all the X11 stuff off into its own file(s) so that
- different backends / font backends can be easily introduced in the future. */
-cairo_surface_t *cairo_quartz_surface_create(void *ctx, int width, int height);
-
 // coverity[+alloc : arg-*3]
 GpStatus
 GdipCreateFromQuartz_macosx (void *ctx, int width, int height, GpGraphics **graphics)
 {
-        cairo_surface_t *surface;
+	cairo_surface_t *surface;
 
 	if (!graphics)
 		return InvalidParameter;
 
-	surface = cairo_quartz_surface_create(ctx, width, height);
+	surface = cairo_quartz_surface_create (0, width, height);
 
 	*graphics = gdip_graphics_new(surface);
 	(*graphics)->dpi_x = (*graphics)->dpi_y = gdip_get_display_dpi ();
@@ -279,6 +273,7 @@ GdipCreateFromQuartz_macosx (void *ctx, int width, int height, GpGraphics **grap
 	(*graphics)->bounds.Height = height;
 
 	(*graphics)->type = gtOSXDrawable;
+	(*graphics)->cg_context = ctx;
 
 	return Ok;
 }
@@ -941,55 +936,6 @@ GdipDrawLinesI (GpGraphics *graphics, GpPen *pen, GDIPCONST GpPoint *points, int
 	}
 }
 
-static GpStatus
-gdip_plot_path (GpGraphics *graphics, GpPath *path, BOOL antialiasing)
-{
-        int length = path->count;
-        int i, idx = 0;
-
-        for (i = 0; i < length; ++i) {
-                GpPointF pt = g_array_index (path->points, GpPointF, i);
-                BYTE type = g_array_index (path->types, BYTE, i);
-                GpPointF pts [3];
-
-		/* mask the bits so that we get only the type value not the other flags */
-                switch (type & PathPointTypePathTypeMask) {
-                case PathPointTypeStart:
-			gdip_cairo_move_to (graphics, pt.X, pt.Y, TRUE, antialiasing);
-                        break;
-
-                case PathPointTypeLine:
-			gdip_cairo_line_to (graphics, pt.X, pt.Y, TRUE, antialiasing);
-                        break;
-
-                case PathPointTypeBezier:
-                        /* make sure we only add at most 3 points to pts */
-                        if (idx < 3) {
-                                pts [idx] = pt;
-                                idx ++;
-                        }
-
-                        /* once we've added 3 pts, we can draw the curve */
-                        if (idx == 3) {
-				gdip_cairo_curve_to (graphics, pts[0].X, pts[0].Y, pts[1].X, pts[1].Y,
-					pts[2].X, pts[2].Y, TRUE, antialiasing);
-
-                                idx = 0;
-                        }
-
-                        break;
-                default:
-			g_warning ("Unknown PathPointType %d", type);
-                        return NotImplemented;
-                }
-
-		/* close the subpath */
-		if (type & PathPointTypeCloseSubpath)
-			cairo_close_path (graphics->ct);
-        }
-	return Ok;
-}
-
 GpStatus
 GdipDrawPath (GpGraphics *graphics, GpPen *pen, GpPath *path)
 {
@@ -1858,6 +1804,20 @@ GdipFlush (GpGraphics *graphics, GpFlushIntention intention)
 
 	surface = cairo_get_target (graphics->ct);
 	cairo_surface_flush (surface);
+
+#ifdef CAIRO_HAS_QUARTZ_SURFACE
+	if (graphics->type == gtOSXDrawable) {
+		CGRect rect;
+	
+		rect.origin.x = 0;
+		rect.origin.y = 0;
+		rect.size.width = graphics->bounds.Width;
+		rect.size.height = graphics->bounds.Height;
+		void *image = CGBitmapContextCreateImage (cairo_quartz_surface_get_cg_context (surface));
+		CGContextDrawImage (graphics->cg_context, rect, image);
+		CGImageRelease (image);
+	}
+#endif
 	return Ok;
 }
 
