@@ -28,10 +28,10 @@
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include "gdiplus-private.h"
 
-#include <stdio.h>
-#include "jpegcodec.h"
-#include "gdipImage.h"
+GUID gdip_jpg_image_format_guid = {0xb96b3caeU, 0x0728U, 0x11d3U, {0x9d, 0x7b, 0x00, 0x00, 0xf8, 0x1e, 0xf3, 0x2e}};
+extern GUID GdipEncoderQuality;
 
 #ifdef HAVE_LIBJPEG
 
@@ -47,6 +47,7 @@
 #include <libexif/exif-entry.h>
 #endif
 
+#include "jpegcodec.h"
 
 /* Codecinfo related data*/
 static ImageCodecInfo jpeg_codec;
@@ -69,7 +70,7 @@ gdip_getcodecinfo_jpeg ()
 	jpeg_codec.FormatDescription = (const WCHAR*) jpeg_format;
 	jpeg_codec.FilenameExtension = (const WCHAR*) jpeg_extension;
 	jpeg_codec.MimeType = (const WCHAR*) jpeg_mimetype;
-	jpeg_codec.Flags = Encoder | Decoder | SupportBitmap | Builtin;
+	jpeg_codec.Flags = ImageCodecFlagsEncoder | ImageCodecFlagsDecoder | ImageCodecFlagsSupportBitmap | ImageCodecFlagsBuiltin;
 	jpeg_codec.Version = 1;
 	jpeg_codec.SigCount = 1;
 	jpeg_codec.SigSize = 2;
@@ -153,7 +154,7 @@ _gdip_source_dummy_init (j_decompress_ptr cinfo)
 	/* nothing */
 }
 
-static bool
+static BOOL
 _gdip_source_stdio_fill_input_buffer (j_decompress_ptr cinfo)
 {
 	gdip_stdio_jpeg_source_mgr_ptr src = (gdip_stdio_jpeg_source_mgr_ptr) cinfo->src;
@@ -194,7 +195,7 @@ _gdip_source_stdio_skip_input_data (j_decompress_ptr cinfo, long skipbytes)
 	}
 }
 
-static bool
+static BOOL
 _gdip_source_stream_fill_input_buffer (j_decompress_ptr cinfo)
 {
 	gdip_stream_jpeg_source_mgr_ptr src = (gdip_stream_jpeg_source_mgr_ptr) cinfo->src;
@@ -276,9 +277,9 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 	struct jpeg_decompress_struct	cinfo;
 	struct gdip_jpeg_error_mgr	jerr;
 	GpBitmap	*result;
-	guchar		*destbuf;
-	guchar		*destptr;
-	guchar		*lines[4] = {NULL, NULL, NULL, NULL};
+	BYTE		*destbuf;
+	BYTE		*destptr;
+	BYTE		*lines[4] = {NULL, NULL, NULL, NULL};
 	GpStatus	status;
 	int		stride;
 
@@ -304,9 +305,10 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 	cinfo.do_block_smoothing = FALSE;
 
 	result = gdip_bitmap_new_with_frame (NULL, TRUE);
-	result->type = imageBitmap;
+	result->type = ImageTypeBitmap;
 	result->active_bitmap->width = cinfo.image_width;
 	result->active_bitmap->height = cinfo.image_height;
+	result->active_bitmap->image_flags = ImageFlagsReadOnly | ImageFlagsHasRealPixelSize | ImageFlagsPartiallyScalable | ImageFlagsHasRealDPI;
 
 	if (cinfo.density_unit == 1) { /* dpi */
 		result->active_bitmap->dpi_horz = cinfo.X_density;
@@ -320,44 +322,44 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 	}
 
 	if (cinfo.num_components == 1) {
-		result->active_bitmap->pixel_format = Format8bppIndexed;
+		result->cairo_format = CAIRO_FORMAT_A8;
+		result->active_bitmap->stride = cinfo.image_width;
+		result->active_bitmap->pixel_format = PixelFormat8bppIndexed;
 	} else if (cinfo.num_components == 3) {
 		/* libjpeg gives us RGB for many formats and
 		 * we convert to RGB format when needed. JPEG
 		 * does not support alpha (transparency). */
-		result->active_bitmap->pixel_format = Format24bppRgb;
+		result->cairo_format = CAIRO_FORMAT_ARGB32;
+		result->active_bitmap->stride = 4 * cinfo.image_width;
+		result->active_bitmap->pixel_format = PixelFormat24bppRGB;
 	} else if (cinfo.num_components == 4) {
-		result->active_bitmap->pixel_format = Format32bppRgb;
+		result->cairo_format = CAIRO_FORMAT_ARGB32;
+		result->active_bitmap->stride = 4 * cinfo.image_width;
+		result->active_bitmap->pixel_format = PixelFormat32bppRGB;
 	}
 
 	switch (cinfo.jpeg_color_space) {
-		case JCS_GRAYSCALE:
-			result->active_bitmap->image_flags = ImageFlagsColorSpaceGRAY;
-			break;
-
-		case JCS_RGB:
-			result->active_bitmap->image_flags = ImageFlagsColorSpaceRGB;
-			break;
-
-		case JCS_YCbCr:
-			result->active_bitmap->image_flags = ImageFlagsColorSpaceYCBCR;
-			break;
-
-		case JCS_YCCK:
-			result->active_bitmap->image_flags = ImageFlagsColorSpaceYCCK;
-			break;
-
-		case JCS_CMYK:
-			result->active_bitmap->image_flags = ImageFlagsColorSpaceCMYK;
-			break;
-
-		default:
-			result->active_bitmap->image_flags = ImageFlagsNone; /* Unknown Colorspace */
+	case JCS_GRAYSCALE:
+		result->active_bitmap->image_flags |= ImageFlagsColorSpaceGRAY;
+		if (cinfo.num_components == 1)
+			result->active_bitmap->palette = gdip_create_greyscale_palette (256);
+		break;
+	case JCS_RGB:
+		result->active_bitmap->image_flags |= ImageFlagsColorSpaceRGB;
+		break;
+	case JCS_YCbCr:
+		result->active_bitmap->image_flags |= ImageFlagsColorSpaceYCBCR;
+		break;
+	case JCS_YCCK:
+		result->active_bitmap->image_flags |= ImageFlagsColorSpaceYCCK;
+		break;
+	case JCS_CMYK:
+		result->active_bitmap->image_flags |= ImageFlagsColorSpaceCMYK;
+		break;
+	default:
+		break;
 	}
 
-	result->cairo_format = CAIRO_FORMAT_ARGB32;
-	result->active_bitmap->stride = 4 * cinfo.image_width;
-
 	stride = result->active_bitmap->stride;
 
 	/* Request cairo-compat output */
@@ -368,13 +370,26 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 	 * YCCK to CMYK using the libjpeg. We convert CMYK
 	 * to RGB ourself.
 	 */
-	if (cinfo.jpeg_color_space == JCS_RGB || cinfo.jpeg_color_space == JCS_YCbCr || cinfo.jpeg_color_space == JCS_GRAYSCALE) {
+	switch (cinfo.jpeg_color_space) {
+	case JCS_GRAYSCALE:
+		/* special case for indexed 256 greyscale images (bug #81552) */
+		if (cinfo.num_components == 1) {
+			cinfo.out_color_space = JCS_GRAYSCALE;
+			cinfo.out_color_components = 1;
+			break;
+		}
+		/* else treat as RGB */
+	case JCS_RGB:
+	case JCS_YCbCr:
 		cinfo.out_color_space = JCS_RGB;
 		cinfo.out_color_components = 3;
-	} else if (cinfo.jpeg_color_space == JCS_YCCK ||cinfo.jpeg_color_space == JCS_CMYK) {
+		break;
+	case JCS_YCCK:
+	case JCS_CMYK:
 		cinfo.out_color_space = JCS_CMYK;
 		cinfo.out_color_components = 4;
-	} else {
+		break;
+	default:
 		/* Unsupported JPEG color space */
 		status = InvalidParameter;
 		goto error;
@@ -404,9 +419,7 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 			int i, j;
 
 			for (i = 0; i < cinfo.rec_outbuf_height; i++) {
-				guchar *lineptr;
-
-				lineptr = lines [i];
+				BYTE *lineptr = lines [i];
 
 				for (j = 0; j < cinfo.output_width; j++) {
 					JOCTET c, m, y, k;
@@ -433,11 +446,13 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 					lineptr += 4;
 				}
 			}
+		} else if (cinfo.out_color_components == 1) {
+			/* no decoding required, we already have all we need */
 		} else {
+			int width = result->active_bitmap->width;
 			for (i = 0; i < nlines; i++) {
 				int j;
-				guchar *inptr, *outptr;
-				int width = result->active_bitmap->width;
+				BYTE *inptr, *outptr;
 
 				inptr = lines[i] + (width * 3);
 				outptr = lines[i] + stride;
@@ -451,7 +466,7 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 				}
 				/* keep last 2 lines in temporary variables */
 				if (width > 1) {
-					byte b2, g2, r2, b1, g1, r1;
+					BYTE b2, g2, r2, b1, g1, r1;
 					inptr -= 3;
 					b2 = inptr[2];
 					g2 = inptr[1];
@@ -466,7 +481,7 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 					set_pixel_bgra(outptr, 0, b1, g1, r1, 0xff);
 				} else 	if (width > 0) {
 					/* in case the jpeg has a single line */
-					byte b, g, r;
+					BYTE b, g, r;
 					inptr -= 3;
 					b = inptr[2];
 					g = inptr[1];
@@ -484,13 +499,9 @@ gdip_load_jpeg_image_internal (struct jpeg_source_mgr *src, GpImage **image)
 	result->active_bitmap->scan0 = destbuf;
 	result->active_bitmap->reserved = GBD_OWN_SCAN0;
 
-	result->surface = cairo_image_surface_create_for_data ((unsigned char *)destbuf, result->cairo_format,
+	result->surface = cairo_image_surface_create_for_data ((BYTE*)destbuf, result->cairo_format,
 		result->active_bitmap->width, result->active_bitmap->height, stride);
 
-	/* win32 returns this as PartiallyScalable and ColorSpaceYCBCR; we
-	 * just return it as RGB (or Grayscale). */
-	result->active_bitmap->image_flags |= ImageFlagsReadOnly | ImageFlagsHasRealPixelSize;
-
 	*image = result;
 	return Ok;
 
@@ -536,7 +547,7 @@ load_exif_data (ExifData *exif_data, GpImage *image)
 	exif_data_foreach_content (exif_data, add_properties_from_content, bitmap);
 	/* thumbnail */
 	if (exif_data->size != 0) {
-		gdip_bitmapdata_property_add (bitmap, ThumbnailData, exif_data->size, TypeByte, exif_data->data);
+		gdip_bitmapdata_property_add (bitmap, PropertyTagThumbnailData, exif_data->size, PropertyTagTypeByte, exif_data->data);
 	}
 	exif_data_unref (exif_data);
 }
@@ -591,7 +602,7 @@ gdip_load_jpeg_image_from_stream_delegate (dstream_t *loader, GpImage **image)
 	GpStatus st;
 #ifdef HAVE_LIBEXIF
 	unsigned int length;
-	unsigned char *ptr;
+	BYTE *ptr;
 #endif
 
 	gdip_stream_jpeg_source_mgr_ptr src;
@@ -638,18 +649,17 @@ gdip_save_jpeg_image_internal (FILE *fp, PutBytesDelegate putBytesFunc, GpImage
 	int		need_argb_conversion = 0;
 	GpStatus	status;
 
-
 	/* Verify that we can support this pixel format */
 	switch (image->active_bitmap->pixel_format) {
-		case Format32bppArgb:
-		case Format32bppPArgb:
-		case Format32bppRgb:
-		case Format24bppRgb:
+		case PixelFormat32bppARGB:
+		case PixelFormat32bppPARGB:
+		case PixelFormat32bppRGB:
+		case PixelFormat24bppRGB:
 			break;
 
-		case Format1bppIndexed:
-		case Format4bppIndexed:
-		case Format8bppIndexed:
+		case PixelFormat1bppIndexed:
+		case PixelFormat4bppIndexed:
+		case PixelFormat8bppIndexed:
 			image = gdip_convert_indexed_to_rgb (image);
 			if (image == NULL) {
 				return OutOfMemory;
@@ -694,7 +704,7 @@ gdip_save_jpeg_image_internal (FILE *fp, PutBytesDelegate putBytesFunc, GpImage
 	if (gdip_get_pixel_format_components (image->active_bitmap->pixel_format) == 3) {
 		cinfo.in_color_space = JCS_RGB;
 		cinfo.input_components = 3;
-		if (image->active_bitmap->pixel_format == Format24bppRgb) {
+		if (image->active_bitmap->pixel_format == PixelFormat24bppRGB) {
 			need_argb_conversion = 1;
 		} else {
 			need_argb_conversion = 0;
@@ -728,7 +738,7 @@ gdip_save_jpeg_image_internal (FILE *fp, PutBytesDelegate putBytesFunc, GpImage
 
 				quality = (pval[0] + pval[1]) / 2;
 			} else if (param->Type == EncoderParameterValueTypeByte) {
-				quality = *(unsigned char *)param->Value;
+				quality = *(BYTE*)param->Value;
 			} else if (param->Type == EncoderParameterValueTypeShort) {
 				quality = *(short *)param->Value;
 			} else {
@@ -743,7 +753,7 @@ gdip_save_jpeg_image_internal (FILE *fp, PutBytesDelegate putBytesFunc, GpImage
 	jpeg_start_compress (&cinfo, TRUE);
 
 	if (need_argb_conversion) {
-		guchar *inptr, *outptr;
+		BYTE *inptr, *outptr;
 		int i, j;
 
 		scanline = GdipAlloc (image->active_bitmap->stride);
@@ -830,6 +840,9 @@ gdip_save_jpeg_image_to_stream_delegate (PutBytesDelegate putBytesFunc,
 
 /* No libjpeg */
 
+#include "image.h"
+#include "dstream.h"
+
 ImageCodecInfo *
 gdip_getcodecinfo_jpeg ()
 {
@@ -891,7 +904,7 @@ gdip_get_encoder_parameter_list_size_jpeg ()
 GpStatus
 gdip_fill_encoder_parameter_list_jpeg (EncoderParameters *eps, UINT size)
 {
-	unsigned char *ucptr = (unsigned char *) eps;
+	BYTE *ucptr = (BYTE*) eps;
 	int *iptr;
 
 	if (!eps || (size < gdip_get_encoder_parameter_list_size_jpeg ()) || ((size & 3) != 0))
