@@ -24,6 +24,9 @@
  */
 
 #include "gdiplus-private.h"
+
+#ifndef USE_PANGO_RENDERING
+
 #include "text-cairo-private.h"
 #include "graphics-private.h"
 #include "graphics-cairo-private.h"
@@ -47,11 +50,9 @@ CalculateStringWidths (cairo_t *ct, GDIPCONST GpFont *gdiFont, GDIPCONST gunicha
 	CurrentDetail = StringDetails;
 
 	for (i = 0; i < StringDetailElements; i++) {
-		if ((CurrentDetail->Flags & STRING_DETAIL_LF) == 0) {
-			utf8[utf8_encode_ucs2char(*(stringUnicode + i), utf8)] = '\0';
-			cairo_text_extents(ct, (const char *) utf8, &ext);
-			CurrentDetail->Width = ext.x_advance;
-		}
+		utf8[utf8_encode_ucs2char(*(stringUnicode + i), utf8)] = '\0';
+		cairo_text_extents(ct, (const char *) utf8, &ext);
+		CurrentDetail->Width = ext.x_advance;
 		CurrentDetail++;
 	}
 
@@ -211,10 +212,7 @@ MeasureString (GpGraphics *graphics, GDIPCONST WCHAR *stringUnicode, int *length
 	cairo_set_font_face (graphics->ct, (cairo_font_face_t*) font->cairofnt);	/* Set our font; this will also be used for later drawing */
 
 	/* this will always return the same value, except when printing */
-	if (graphics->type == gtPostScript)
-		FontSize = gdip_unit_conversion (UnitPixel, UnitCairoPoint, gdip_get_display_dpi (), graphics->type, font->sizeInPixels);
-	else
-		FontSize = font->sizeInPixels;
+	FontSize = (graphics->type == gtPostScript) ? font->emSize : font->sizeInPixels;
 	cairo_set_font_size (graphics->ct, FontSize);
 	
 	cairo_font_extents (graphics->ct, &FontExtent);		/* Get the size we're looking for */
@@ -235,11 +233,13 @@ MeasureString (GpGraphics *graphics, GDIPCONST WCHAR *stringUnicode, int *length
 
 	Src=stringUnicode;
 
-	/* Handle trailing spaces like MS GDI+, see http://bugzilla.ximian.com/show_bug.cgi?id=80680 */
-	while ((StringLen > 0) && (isspace (*(Src + StringLen - 1))))
-		StringLen--;
-	if (StringLen == 0)
-		StringLen = 1;
+	/* unless specified we don't consider the trailing spaces, unless there is just one space (#80680) */
+	if ((format->formatFlags & StringFormatFlagsMeasureTrailingSpaces) == 0) {
+		while ((StringLen > 0) && (isspace (*(Src + StringLen - 1))))
+			StringLen--;
+		if (StringLen == 0)
+			StringLen = 1;
+	}
 
 	Dest=CleanString;
 	CurrentDetail=StringDetails;
@@ -747,6 +747,7 @@ MeasureString (GpGraphics *graphics, GDIPCONST WCHAR *stringUnicode, int *length
 		data->align_vert = AlignVert;
 		data->line_height = LineHeight;
 		data->max_y = MaxY;
+		data->descent = FontExtent.descent;
 	}
 
 	return Ok;
@@ -1127,7 +1128,7 @@ cairo_MeasureCharacterRanges (GpGraphics *graphics, GDIPCONST WCHAR *stringUnico
 	if (status != Ok)
 		goto cleanup;
 
-	lineHeight = data.line_height;
+	lineHeight = data.line_height + data.descent;
 
 	/* Create a region for every char range */
 	for (i = 0; i < format->charRangeCount; i++) {
@@ -1161,6 +1162,10 @@ cairo_MeasureCharacterRanges (GpGraphics *graphics, GDIPCONST WCHAR *stringUnico
 				continue;
 			}
 
+			/* workaround the fact that current implementation thinks LF is on the next line */
+			if ((j == end - 1) && (StringDetails[j].Flags & STRING_DETAIL_LF))
+				continue;
+
 			if (format->formatFlags & StringFormatFlagsDirectionVertical) {
 				charRect.X = StringDetails [j].PosY;
 				charRect.Y = StringDetails [j].PosX;
@@ -1195,3 +1200,5 @@ cleanup:
 
 	return status;
 }
+
+#endif
